﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using System.Windows.Forms;
using Nevron;
using Nevron.Chart;
using Nevron.Dom;
using Nevron.GraphicsCore;

namespace WindowsFormsApplication1
{
	public partial class Form1 : Form
	{
		private const int _chartHeight = 300;

		public Form1()
		{
			InitializeComponent();

			RegisterNevron();
			InitializeChart();
		}

		private static void RegisterNevron()
		{
			//Put a license code here
			var license = new NLicense("");
			NLicenseManager.Instance.SetLicense(license);
			NLicenseManager.Instance.LockLicense = true;
		}

		private void InitializeChart()
		{
			var container = CreateChartContainer(_chartHeight);
			var title = CreateChartTitle("Chart Title");
			var chart = CreateChart();

			container.ChildPanels.Add(title);
			container.ChildPanels.Add(chart);

			chartControl.Panels.Clear();
			chartControl.Panels.Add(container);

			var verticalAxis = CreateVerticalAxis(chart, "A very long text in the vertical axis. It should use several lines");
			CreateSeries(chart, verticalAxis);
		}

		private static NDockPanel CreateChartContainer(int height)
		{
			return new NDockPanel
			{
				DockMode = PanelDockMode.Top,
				Size = new NSizeL(new NLength(0), new NLength(height, NGraphicsUnit.Pixel))
			};
		}

		private static NLabel CreateChartTitle(string text)
		{
			return new NLabel
			{
				Text = text,
				DockMode = PanelDockMode.Top,
				TextStyle =
				{
					FontStyle = { Style = FontStyle.Bold },
					StringFormatStyle = { VertAlign = VertAlign.Top, HorzAlign = HorzAlign.Center }
				},
				Location = new NPointL(new NLength(50f, NRelativeUnit.ParentPercentage), new NLength(2f, NRelativeUnit.ParentPercentage)),
				ContentAlignment = ContentAlignment.BottomCenter
			};
		}

		private static NCartesianChart CreateChart()
		{
			var chart = new NCartesianChart
			{
				BoundsMode = BoundsMode.Stretch,
				DockMode = PanelDockMode.Fill, // This seems to be essential in fixing tooltips misalignment (FB #2425)
				Margins = new NMarginsL(new NLength(15))
			};

			chart.Axis(StandardAxis.PrimaryY).Visible = false;
			chart.Axis(StandardAxis.SecondaryY).Visible = false;
			chart.Axis(StandardAxis.PrimaryX).ScaleConfigurator = new NDateTimeScaleConfigurator();
			chart.Axis(StandardAxis.PrimaryX).ScrollBar.Visible = true;

			var xAxis = chart.Axis(StandardAxis.PrimaryX);
			var xScaleConfigurator = (NDateTimeScaleConfigurator)xAxis.ScaleConfigurator;
			xScaleConfigurator.EnableUnitSensitiveFormatting = false;
			xScaleConfigurator.LabelStyle.Angle = new NScaleLabelAngle(ScaleLabelAngleMode.Scale, 90);
			xScaleConfigurator.LabelValueFormatter = new NDateTimeValueFormatter("HH:mm dd-MMM-yyyy");

			return chart;
		}
		private static NCartesianAxis CreateVerticalAxis(NCartesianChart nChart, string text)
		{
			var nAxis = ((NCartesianAxisCollection)nChart.Axes).AddCustomAxis(AxisOrientation.Vertical, AxisDockZone.FrontLeft);
			nAxis.Anchor = new NDockAxisAnchor(AxisDockZone.FrontLeft, true);
			nAxis.ScrollBar.Visible = true;
			nAxis.PagingView.ResetMode = PagingViewResetMode.View;
			nAxis.UseRangeInflateWhenZoomed = true;

			//nAxis.ScaleConfigurator = CreateVerticalAxisScaleConfigurator(text, Color.Black, Color.Black, Color.Black);
			CreateVerticalAxisLabel(nAxis, text);
			return nAxis;
		}

		private static NLinearScaleConfigurator CreateVerticalAxisScaleConfigurator(string title, Color axisColor, Color labelColor, Color titleColor)
		{
			return new NLinearScaleConfigurator
			{
				Title = { Text = title, TextStyle = { FillStyle = new NColorFillStyle(titleColor) } },
				InnerMajorTickStyle = { LineStyle = { Color = axisColor } },
				OuterMajorTickStyle = { LineStyle = { Color = axisColor } },
				LabelStyle = { TextStyle = { FillStyle = new NColorFillStyle(labelColor) } },
				RulerStyle = { FillStyle = new NColorFillStyle(axisColor), BorderStyle = { Color = axisColor } },
				InflateViewRangeBegin = true,
				InflateViewRangeEnd = true
			};
		}

		private static void CreateVerticalAxisLabel(NCartesianAxis nAxis, string text)
		{
			// How to create axis label with word wrap support
			// https://www.nevron.com/Forum/PrintTopic7922.aspx

			nAxis.UpdateScale();

			var style = new NRangeScaleLabelStyle();
			style.WrapText = true;
			style.TextStyle.TextFormat = TextFormat.Simple;
			style.TickMode = RangeLabelTickMode.None;
			style.Angle = new NScaleLabelAngle(ScaleLabelAngleMode.View, 90);

			var anchor = new NRulerRangeDecorationAnchor(HorzAlign.Left, new NLength(0), HorzAlign.Right, new NLength(0));
			var label = new NRangeScaleLabel(anchor, text, style);

			var decorator = new NCustomScaleDecorator();
			decorator.Decorations.Add(label);

			var scaleLevel = new NScaleLevel();
			scaleLevel.Decorators.Add(decorator);
			nAxis.Scale.Levels.Add(scaleLevel);
		}

		private void CreateSeries(NCartesianChart nChart, NAxis yAxis)
		{
			var mainSeries = CreateNewEmptyLineSeries(nChart, 1, Color.Red, true, true);
			ConfigureSeriesData(mainSeries, yAxis);
		}

		private static NLineSeriesBase CreateNewEmptyLineSeries(NCartesianChart nChart, float thickness, Color color, bool visible, bool showOnLegend)
		{
			var nSeries = (NLineSeriesBase)nChart.Series.Add(SeriesType.Line);
			nSeries.BorderStyle = new NStrokeStyle(thickness, color, LinePattern.Solid);
			nSeries.Legend.Mode = showOnLegend ? SeriesLegendMode.Series : SeriesLegendMode.None;
			nSeries.Visible = visible;
			return nSeries;
		}

		private static void ConfigureSeriesData(NXYScatterSeries nSeries, NAxis yAxis)
		{
			var data = GenerateData(100).ToList();
			nSeries.UseXValues = true;
			nSeries.XValues.AddRange(data.Select(x => x.Item1).ToList());
			nSeries.Values.AddRange(data.Select(x => x.Item2).ToList());
			nSeries.DisplayOnAxis(StandardAxis.PrimaryY, false);
			nSeries.DisplayOnAxis(yAxis.AxisId, true);
		}

		private static IEnumerable<Tuple<DateTime, decimal>> GenerateData(int count)
		{
			var now = DateTime.Now;
			var random = new Random();

			for (var i = 0; i < count; i++)
				yield return Tuple.Create(now.AddDays(i), (decimal)random.Next(1, 50));
		}
	}
}